library lokalise_flutter_sdk;

import 'package:lokalise_flutter_sdk/src/ota/domain/models/update_result.dart';
import 'package:lokalise_flutter_sdk/src/ota/helpers/initializer.dart';

abstract class Lokalise {
  static Lokalise? _instance;

  /// Get the Lokalise singleton instance
  static Lokalise get instance => Lokalise._instance!;

  /// Init the SDK
  static Future<void> init({
    required String projectId,
    required String sdkToken,
    String appVersion = '',
    bool preRelease = false,
  }) async {
    assert(projectId.trim().isNotEmpty, 'Lokalise projectId cannot be empty');
    assert(sdkToken.trim().isNotEmpty, 'Lokalise sdkToken cannot be empty');
    final instance = await Initializer(
      projectId: projectId,
      sdkToken: sdkToken,
      appVersion: appVersion,
      preRelease: preRelease,
    ).lokalise;
    await instance.load();
    _instance = instance;
  }

  /// Checks the latest remote bundle or the one assigned to
  /// the app version following the bundle freeze settings
  /// and update the local translations with it if needed.
  Future<UpdateResult> update();

  /// Internal method used in l10n.dart to set the translations metadata
  set metadata(Map<String, List<String>> metadata);
}
