import 'package:lokalise_flutter_sdk/src/ota/data/repositories/local_bundle_repository.dart';
import 'package:lokalise_flutter_sdk/src/ota/data/repositories/remote_bundle_repository.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/dto/get_up_to_date_bundle_dto.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/exceptions/bundle_not_found_exception.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/exceptions/lokalise_exception.dart';
import 'package:lokalise_flutter_sdk/src/ota/domain/models/bundle.dart';

class GetUpToDateBundleUseCase {
  final RemoteBundleRepository _remoteRepo;
  final LocalBundleRepository _localRepo;

  GetUpToDateBundleUseCase({
    required RemoteBundleRepository remoteBundleRepository,
    required LocalBundleRepository localBundleRepository,
  })  : _remoteRepo = remoteBundleRepository,
        _localRepo = localBundleRepository;

  Future<Bundle> getBundle({required GetUpToDateBundleDto dto}) async {
    try {
      Bundle? bundle = await _remoteRepo.getBundle(getUpToDateBundleDto: dto);

      if (bundle != null &&
          bundle.translationVersion != dto.translationVersion) {
        await _localRepo.saveBundle(bundle: bundle);
      } else {
        bundle = _localRepo.getBundle() ??
            Bundle.empty(
              projectId: dto.credentials.projectId,
              appVersion: dto.appVersion,
            );
      }

      return bundle;
    } on BundleNotFoundException {
      await _localRepo.removeBundle();
      return Bundle.empty(
        projectId: dto.credentials.projectId,
        appVersion: dto.appVersion,
      );
    } catch (_) {
      // in the future we will send the error to our server
      throw LokaliseException('Unexpected exception');
    }
  }
}
