import 'package:flutter/material.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:lokalise_flutter_sdk/lokalise_flutter_sdk.dart';
import 'package:lokalise_flutter_sdk/src/ota/lokalise_v2.dart';
import 'package:package_info_plus/package_info_plus.dart';
import 'package:shared_preferences/shared_preferences.dart';

void main() {
  setUp(() {
    WidgetsFlutterBinding.ensureInitialized();
    SharedPreferences.setMockInitialValues({});
    PackageInfo.setMockInitialValues(
      appName: 'test',
      packageName: 'test',
      version: 'test.version',
      buildNumber: 'test',
      buildSignature: 'test',
      installerStore: 'test',
    );
  });

  group('Lokalise - init', () {
    test('invalid project id', () async {
      expect(
        Lokalise.init(projectId: '', sdkToken: 'token'),
        throwsA(const TypeMatcher<AssertionError>()),
      );
      expect(
        Lokalise.init(projectId: '    ', sdkToken: 'token'),
        throwsA(const TypeMatcher<AssertionError>()),
      );
    });

    test('invalid sdk token', () async {
      expect(
        Lokalise.init(projectId: 'project', sdkToken: ''),
        throwsA(const TypeMatcher<AssertionError>()),
      );
      expect(
        Lokalise.init(projectId: 'project', sdkToken: '     '),
        throwsA(const TypeMatcher<AssertionError>()),
      );
    });

    test('works - just required parameters', () async {
      await Lokalise.init(projectId: 'project', sdkToken: 'token');
      expect(Lokalise.instance, isA<LokaliseV2>());
    });

    test('works - all parameters', () async {
      await Lokalise.init(
        projectId: 'project',
        sdkToken: 'token',
        appVersion: '1.0.0',
        preRelease: true,
      );
      expect(Lokalise.instance, isA<LokaliseV2>());
    });
  });
}
